/* $Id: debug.h,v 1.1 2000/03/13 04:51:41 ajapted Exp $
******************************************************************************

   LibGFFD - debugging framework

   Copyright (C) 1998-1999  Marcus Sundberg  [marcus@ggi-project.org]
   Copyright (C) 2000  Andrew Apted    [andrew.apted@ggi-project.org]
  
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
 
   Notes:
   ------

   All debug messages can be completely suppressed (and no code is
   generated for them) by undefining DEBUG in the Makerules.

   Some things need to be #defined before #including this file :

     o  GD_DEBUG_PKG must be a string containing the package name
        (e.g. "libgffd").  This is mandatory.  

     o  GD_DEBUG_LEVEL should be defined as the per-file debug
        level; defaults to 0.

     o  GD_DEBUG_FILE should be defined as a string containing the
        full pathname of the current file (e.g. "main/init.c").  It
        defaults to the value of __FILE__.
    
   The following macros/functions are provided :

     o  GD_NOTICE(format, ...)   Show a plain message.

     o  GD_ERROR(format, ...)    Show an error message.
     
     o  GD_DEBUG#(format, ...)   Show a debugging message.  # is a
                                 number from 1 to 3 for the debug
                                 level.
     
     .  GD_ASSERT(expression)    When the expression is false, show an
                                 error message and halt the program.
 
******************************************************************************
*/

#ifndef _GFFD_INTERNAL_DEBUG_H
#define _GFFD_INTERNAL_DEBUG_H


#ifndef GD_DEBUG_PKG
#error GD_DEBUG_PKG not defined
#endif

#ifndef GD_DEBUG_LEVEL
#define GD_DEBUG_LEVEL  0
#endif

#ifndef GD_DEBUG_FILE
#define GD_DEBUG_FILE  __FILE__
#endif


__BEGIN_DECLS

void gffdDPrintf(const char *prefix, const char *format, ...);

__END_DECLS


/* do nothing error macro */

#ifdef __GNUC__
#define GD_IGNORE(format, args...)  do {} while(0)
#else
static inline void GD_IGNORE(const char *format, ...) {}
#endif


/* notice, error and base debugging macros */

#ifdef __GNUC__

#define GD_NOTICE(format, args...)    \
    do { gffdDPrintf(GD_DEBUG_PKG ": ",  \
       format, ##args); } while(0)

#define GD_ERROR(format, args...)  \
    do { gffdDPrintf(GD_DEBUG_PKG " ERROR: ",  \
       format, ##args); } while(0)

#define GD_DEBUG0(format, args...)  \
    do { gffdDPrintf(GD_DEBUG_PKG " debug (" GD_DEBUG_FILE ") : ",  \
       format, ##args); } while(0)

#else /* __GNUC__ */

__BEGIN_DECLS

static inline void GD_NOTICE(const char *format, ...)
{
#ifdef DEBUG
  char buffer[1024];
  va_list args;

  va_start(args, format);
  vsprintf(buffer, format, args);
  va_end(args);
  
  gffdDPrintf(GD_DEBUG_PKG ": ", "%s", buffer);
#endif
}

static inline void GD_ERROR(const char *format, ...)
{
#ifdef DEBUG
  char buffer[1024];
  va_list args;

  va_start(args, format);
  vsprintf(buffer, format, args);
  va_end(args);
  
  gffdDPrintf(GD_DEBUG_PKG " ERROR: ", "%s", buffer);
#endif
}

static inline void GD_DEBUG0(const char *format, ...)
{
#ifdef DEBUG
  char buffer[1024];
  va_list args;

  va_start(args, format);
  vsprintf(buffer, format, args);
  va_end(args);
  
  gffdDPrintf(GD_DEBUG_PKG " debug (" GD_DEBUG_FILE ") : ", "%s", buffer);
#endif
}

__END_DECLS

#endif /* __GNUC__ */


/* level-sensitive debugging macros */

#if (defined(DEBUG) && (GD_DEBUG_LEVEL >= 1))
#define GD_DEBUG1  GD_DEBUG0
#else
#define GD_DEBUG1  GD_IGNORE
#endif

#if (defined(DEBUG) && (GD_DEBUG_LEVEL >= 2))
#define GD_DEBUG2  GD_DEBUG0
#else
#define GD_DEBUG2  GD_IGNORE
#endif

#if (defined(DEBUG) && (GD_DEBUG_LEVEL >= 3))
#define GD_DEBUG3  GD_DEBUG0
#else
#define GD_DEBUG3  GD_IGNORE
#endif


/* assertion macros */

#ifdef DEBUG

#define GD_ASSERT(expr)  \
   do { if (! (expr)) {  \
       gffdDPrintf(GD_DEBUG_PKG " ERROR: ", "Assertion failed: "  \
           "file %s line %d\n", GD_DEBUG_FILE, __LINE__);  \
       exit(9);  \
   }} while(0)

#else
#define GD_ASSERT(expr)  do {} while(0)
#endif /* DEBUG */


#endif /* _GFFD_INTERNAL_DEBUG_H */
